<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use Illuminate\Validation\Rules\Password;
use Illuminate\Support\Str;

class RegistrationController extends Controller
{
    /**
     * Registra um novo usuário
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function register(Request $request)
    {
        // Validação detalhada dos dados
        $validator = Validator::make($request->all(), [
            'nome' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users,email',
            'password' => [
                'required',
                'string',
                'confirmed',
                Password::min(8)
                    ->letters()
                    ->mixedCase()
                    ->numbers()
                    ->symbols()
            ],
            'empresa' => 'nullable|string|max:255',
            'telefone' => 'nullable|string|max:20',
        ]);

        // Verifica se a validação falhou
        if ($validator->fails()) {
            return response()->json([
                'errors' => $validator->errors()
            ], 422);
        }

        // Tenta criar o usuário
        try {
            // Cria o usuário
            $user = User::create([
                'name' => $request->input('nome'),
                'email' => $request->input('email'),
                'password' => Hash::make($request->input('password')),
                'empresa' => $request->input('empresa'),
                'phone' => $request->input('telefone'),
                'user_type' => 'cliente', // Tipo padrão
                'is_active' => true,
                'remember_token' => Str::random(10)
            ]);

            // Cria token de API
            $token = $user->createToken('API Token')->plainTextToken;

            // Retorna resposta de sucesso
            return response()->json([
                'user' => [
                    'id' => $user->id,
                    'name' => $user->name,
                    'email' => $user->email,
                    'empresa' => $user->empresa,
                    'user_type' => $user->user_type
                ],
                'token' => $token
            ], 201);

        } catch (\Exception $e) {
            // Log do erro para depuração
            \Log::error('Erro ao registrar usuário: ' . $e->getMessage());

            // Retorna erro genérico
            return response()->json([
                'message' => 'Erro ao registrar usuário',
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
