<?php

namespace App\Http\Controllers;

use App\Models\Veiculo;
use App\Models\Frete;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class VeiculoController extends Controller
{
    /**
     * Lista os veículos do usuário autenticado
     */
    public function meusVeiculos(Request $request)
    {
        $user = Auth::user();

        if (!$user) {
            return response()->json(['message' => 'Usuário não autenticado'], 401);
        }

        $query = Veiculo::where('user_id', $user->id);

        // Aplicar ordenação
        $sortColumn = $request->input('sort_column', 'created_at');
        $sortOrder = $request->input('sort_order', 'desc');

        // Garantir que a coluna de ordenação é válida para evitar injeção SQL
        $allowedColumns = [
            'id', 'placa', 'antt', 'tipo_veiculo', 'localizacao_atual',
            'disponivel', 'data_disponibilidade', 'created_at'
        ];

        if (!in_array($sortColumn, $allowedColumns)) {
            $sortColumn = 'created_at';
        }

        $query->orderBy($sortColumn, $sortOrder);

        // Aplicar paginação
        $perPage = $request->input('per_page', 15);
        $veiculos = $query->paginate($perPage);

        // Para cada veículo, adicionar informações de correspondência de fretes
        foreach ($veiculos as $veiculo) {
            $veiculo->fretes_compativeis = $this->getFretesCompativeis($veiculo->id);
        }

        return response()->json([
            'data' => $veiculos->items(),
            'current_page' => $veiculos->currentPage(),
            'per_page' => $veiculos->perPage(),
            'total' => $veiculos->total(),
            'last_page' => $veiculos->lastPage(),
        ]);
    }

    /**
     * Obtém fretes compatíveis para um veículo
     */
    private function getFretesCompativeis($veiculoId)
    {
        $veiculo = Veiculo::findOrFail($veiculoId);

        // Buscar fretes disponíveis com base na localização e tipo de veículo
        $fretes = Frete::where(function($query) use ($veiculo) {
            // Correspondência por origem (localização atual do veículo)
            if ($veiculo->localizacao_atual) {
                $query->where('origem', 'like', '%' . explode(',', $veiculo->localizacao_atual)[0] . '%');
            }

            // Correspondência por tipo de veículo se especificado no frete
            if ($veiculo->tipo_veiculo) {
                $query->orWhere('tipo_veiculo', 'like', '%' . $veiculo->tipo_veiculo . '%')
                      ->orWhereNull('tipo_veiculo');
            }

            // Correspondência por destino de interesse
            if ($veiculo->destino_interesse) {
                $query->orWhere('destino', 'like', '%' . explode(',', $veiculo->destino_interesse)[0] . '%');
            }
        })
        ->orderBy('data_cadastro', 'desc')
        ->limit(5)
        ->get();

        return $fretes;
    }

    /**
     * Exibe um veículo específico
     */
    public function show($id)
    {
        $veiculo = Veiculo::findOrFail($id);

        // Verificar se o usuário é dono do veículo
        if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
            return response()->json([
                'message' => 'Você não tem permissão para visualizar este veículo.'
            ], 403);
        }

        // Adicionar informações de correspondência de fretes
        $veiculo->fretes_compativeis = $this->getFretesCompativeis($veiculo->id);

        return response()->json($veiculo);
    }

    /**
     * Cria um novo veículo
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'placa' => 'required|string|max:15',
            'antt' => 'nullable|string|max:50',
            'tipo_veiculo' => 'required|string|max:100',
            'marca' => 'nullable|string|max:100',
            'modelo' => 'nullable|string|max:100',
            'ano' => 'nullable|integer',
            'capacidade' => 'nullable|string|max:100',
            'rastreado' => 'nullable|boolean',
            'seguro' => 'nullable|boolean',
            'localizacao_atual' => 'nullable|string|max:255',
            'destino_interesse' => 'nullable|string|max:255',
            'disponivel' => 'nullable|boolean',
            'data_disponibilidade' => 'nullable|date',
            'observacoes' => 'nullable|string',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        // Adicionar o ID do usuário atual ao pedido
        $data = $request->all();
        $data['user_id'] = Auth::id();

        // Criar o veículo com o usuário associado
        $veiculo = Veiculo::create($data);

        // Após criar o veículo, atualizar a tabela de compatibilidade
        $this->atualizarCompatibilidade($veiculo->id);

        return response()->json([
            'message' => 'Veículo cadastrado com sucesso!',
            'veiculo' => $veiculo
        ], 201);
    }

    /**
     * Atualiza um veículo existente
     */
    public function update(Request $request, $id)
    {
        $veiculo = Veiculo::findOrFail($id);

        // Verificar se o usuário é dono do veículo
        if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
            return response()->json([
                'message' => 'Você não tem permissão para editar este veículo.'
            ], 403);
        }

        $validator = Validator::make($request->all(), [
            'placa' => 'sometimes|required|string|max:15',
            'antt' => 'nullable|string|max:50',
            'tipo_veiculo' => 'sometimes|required|string|max:100',
            'marca' => 'nullable|string|max:100',
            'modelo' => 'nullable|string|max:100',
            'ano' => 'nullable|integer',
            'capacidade' => 'nullable|string|max:100',
            'rastreado' => 'nullable|boolean',
            'seguro' => 'nullable|boolean',
            'localizacao_atual' => 'nullable|string|max:255',
            'destino_interesse' => 'nullable|string|max:255',
            'disponivel' => 'nullable|boolean',
            'data_disponibilidade' => 'nullable|date',
            'observacoes' => 'nullable|string',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $veiculo->update($request->all());

        // Após atualizar o veículo, atualizar a tabela de compatibilidade
        $this->atualizarCompatibilidade($veiculo->id);

        return response()->json([
            'message' => 'Veículo atualizado com sucesso!',
            'veiculo' => $veiculo
        ]);
    }

    /**
     * Remove um veículo
     */
    public function destroy($id)
    {
        $veiculo = Veiculo::findOrFail($id);

        // Verificar se o usuário é dono do veículo
        if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
            return response()->json([
                'message' => 'Você não tem permissão para excluir este veículo.'
            ], 403);
        }

        $veiculo->delete();

        return response()->json(['message' => 'Veículo removido com sucesso!']);
    }

    /**
     * Atualiza a disponibilidade de um veículo
     */
    public function atualizarDisponibilidade(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'disponivel' => 'required|boolean',
            'data_disponibilidade' => 'nullable|date',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $veiculo = Veiculo::findOrFail($id);

        // Verificar se é um admin ou se é o dono do veículo
        if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
            return response()->json([
                'message' => 'Você não tem permissão para atualizar este veículo.'
            ], 403);
        }

        $veiculo->disponivel = $request->disponivel;

        if ($request->has('data_disponibilidade')) {
            $veiculo->data_disponibilidade = $request->data_disponibilidade;
        }

        $veiculo->save();

        // Após atualizar o veículo, atualizar a tabela de compatibilidade
        if ($veiculo->disponivel) {
            $this->atualizarCompatibilidade($veiculo->id);
        }

        return response()->json([
            'message' => 'Disponibilidade atualizada com sucesso!',
            'veiculo' => $veiculo
        ]);
    }

    /**
     * Atualiza a localização atual de um veículo
     */
    public function atualizarLocalizacao(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'localizacao_atual' => 'required|string|max:255',
            'latitude' => 'nullable|numeric',
            'longitude' => 'nullable|numeric',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $veiculo = Veiculo::findOrFail($id);

        // Verificar se é um admin ou se é o dono do veículo
        if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
            return response()->json([
                'message' => 'Você não tem permissão para atualizar este veículo.'
            ], 403);
        }

        $veiculo->localizacao_atual = $request->localizacao_atual;
        $veiculo->save();

        // Registrar a localização no histórico se latitude e longitude forem fornecidas
        if ($request->has('latitude') && $request->has('longitude')) {
            DB::table('veiculos_localizacoes')->insert([
                'veiculo_id' => $veiculo->id,
                'latitude' => $request->latitude,
                'longitude' => $request->longitude,
                'cidade' => explode(',', $request->localizacao_atual)[0] ?? null,
                'estado' => explode(',', $request->localizacao_atual)[1] ?? null,
                'data_registro' => now()
            ]);
        }

        // Após atualizar o veículo, atualizar a tabela de compatibilidade
        $this->atualizarCompatibilidade($veiculo->id);

        return response()->json([
            'message' => 'Localização atualizada com sucesso!',
            'veiculo' => $veiculo
        ]);
    }

    /**
     * Atualiza a compatibilidade entre veículo e fretes disponíveis
     */
    private function atualizarCompatibilidade($veiculoId)
    {
        $veiculo = Veiculo::findOrFail($veiculoId);

        // Apenas atualizar compatibilidade se o veículo estiver disponível
        if (!$veiculo->disponivel) {
            return;
        }

        // Limpar correspondências antigas
        DB::table('veiculos_fretes_match')->where('veiculo_id', $veiculoId)->delete();

        // Obter fretes compatíveis
        $fretes = $this->getFretesCompativeis($veiculoId);

        // Inserir novas correspondências
        foreach ($fretes as $frete) {
            // Calcular pontuação de compatibilidade simples
            $score = 0;

            // +50 pontos se a origem do frete corresponder à localização atual do veículo
            if ($veiculo->localizacao_atual && stripos($frete->origem, explode(',', $veiculo->localizacao_atual)[0]) !== false) {
                $score += 50;
            }

            // +30 pontos se o destino do frete corresponder ao destino de interesse do veículo
            if ($veiculo->destino_interesse && stripos($frete->destino, explode(',', $veiculo->destino_interesse)[0]) !== false) {
                $score += 30;
            }

            // +20 pontos se o tipo de veículo for compatível
            if ($veiculo->tipo_veiculo &&
                ($frete->tipo_veiculo == null || stripos($frete->tipo_veiculo, $veiculo->tipo_veiculo) !== false)) {
                $score += 20;
            }

            // Inserir na tabela de correspondências
            DB::table('veiculos_fretes_match')->insert([
                'veiculo_id' => $veiculoId,
                'frete_id' => $frete->id,
                'score' => $score,
                'created_at' => now()
            ]);
        }
    }

    /**
     * Retorna os fretes que correspondem a um veículo específico,
     * com suporte para filtro por raio de distância
     */
    public function fretesCompativeis(Request $request, $id)
    {
        try {
            $veiculo = Veiculo::findOrFail($id);

            // Verificar se o usuário é dono do veículo
            if ($veiculo->user_id !== Auth::id() && !Auth::user()->isAdmin()) {
                return response()->json([
                    'message' => 'Você não tem permissão para visualizar os fretes compatíveis deste veículo.'
                ], 403);
            }

            // Obter parâmetros do request com valores padrão seguros
            $maxDistance = $request->input('max_distance', 300); // Default 300km se não especificado
            $origem = $request->input('origem', ''); // Localização atual
            $destino_interesse = $request->input('destino_interesse', ''); // Destino de interesse
            $tipo_veiculo = $request->input('tipo_veiculo', ''); // Tipo de veículo
            $search_all_regions = $request->boolean('search_all_regions', false); // Flag para busca expandida

            // Query base para buscar fretes
            $query = Frete::query();

            // Se não estiver buscando em todas as regiões e tem uma origem, aplica filtros geográficos
            if (!$search_all_regions && !empty($origem)) {
                // Extrair a cidade da origem (assumindo formato "Cidade, UF")
                $parts = explode(',', $origem);
                $cidadeOrigem = trim($parts[0] ?? '');

                // Somente aplicar filtro se tiver uma cidade válida
                if (!empty($cidadeOrigem)) {
                    $query->where(function($q) use ($cidadeOrigem) {
                        // Fretes na mesma cidade
                        $q->where('origem', 'like', "%{$cidadeOrigem}%");

                        // Ou cidades próximas (simplificado - na prática usaria coordenadas)
                        // Para evitar o erro 500, estamos sendo mais cuidadosos com as condições
                    });
                }
            }

            // Filtrar por tipo de veículo, se especificado
            if (!empty($tipo_veiculo)) {
                $query->where(function($q) use ($tipo_veiculo) {
                    $q->where('tipo_veiculo', 'like', "%{$tipo_veiculo}%")
                    ->orWhereNull('tipo_veiculo')
                    ->orWhere('tipo_veiculo', '');
                });
            }

            // Destino de interesse, se especificado
            if (!empty($destino_interesse)) {
                $parts = explode(',', $destino_interesse);
                $cidadeDestino = trim($parts[0] ?? '');

                if (!empty($cidadeDestino)) {
                    $query->orWhere('destino', 'like', "%{$cidadeDestino}%");
                }
            }

            // Ordenar por data de cadastro (mais recentes primeiro)
            $query->orderBy('data_cadastro', 'desc');

            // Limitar resultados para evitar sobrecarga
            $query->limit(50);

            // Executar a consulta e obter os fretes
            $fretes = $query->get();

            // Processar os resultados para adicionar pontuação de compatibilidade
            $fretesProcessados = [];
            foreach ($fretes as $frete) {
                // Iniciar com um score base
                $score = 30;

                // Adicionar score extra para correspondências relevantes
                if (!empty($origem) && !empty($frete->origem)) {
                    $cidadeOrigem = trim(explode(',', $origem)[0] ?? '');
                    if (!empty($cidadeOrigem) && stripos($frete->origem, $cidadeOrigem) !== false) {
                        $score += 40; // Origem na mesma cidade
                    }
                }

                if (!empty($destino_interesse) && !empty($frete->destino)) {
                    $cidadeDestino = trim(explode(',', $destino_interesse)[0] ?? '');
                    if (!empty($cidadeDestino) && stripos($frete->destino, $cidadeDestino) !== false) {
                        $score += 30; // Destino coincide com interesse
                    }
                }

                // Criar objeto frete processado com score
                $fretesProcessados[] = [
                    'id' => $frete->id,
                    'frete_id' => $frete->frete_id,
                    'empresa' => $frete->empresa,
                    'origem' => $frete->origem,
                    'destino' => $frete->destino,
                    'tipo_carga' => $frete->tipo_carga,
                    'tipo_veiculo' => $frete->tipo_veiculo,
                    'distancia' => $frete->distancia,
                    'preco' => $frete->preco,
                    'descricao_pagamento' => $frete->descricao_pagamento,
                    'data_cadastro' => $frete->data_cadastro,
                    'contatos_whatsapp' => $frete->contatos_whatsapp,
                    'contatos_celular' => $frete->contatos_celular,
                    'contatos_fixo' => $frete->contatos_fixo,
                    'observacoes' => $frete->observacoes,
                    'score' => $score
                ];
            }

            // Ordenar por score
            usort($fretesProcessados, function($a, $b) {
                return $b['score'] - $a['score'];
            });

            return response()->json($fretesProcessados);
        }
        catch (\Exception $e) {
            // Log detalhado do erro para facilitar depuração
            \Log::error('Erro ao buscar fretes compatíveis: ' . $e->getMessage(), [
                'veiculo_id' => $id,
                'parameters' => $request->all(),
                'exception' => $e
            ]);

            // Retornar erro 400 com mensagem amigável em vez de 500
            return response()->json([
                'message' => 'Erro ao buscar fretes compatíveis. Por favor, tente novamente.',
                'error' => $e->getMessage()
            ], 400);
        }
    }
}
