<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Auth\Notifications\ResetPassword;
use Illuminate\Support\Facades\URL;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
        'cpf',
        'phone',
        'empresa',
        'user_type',
        'is_active'
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'is_active' => 'boolean',
        'password' => 'hashed',
    ];

    /**
     * Custom password reset notification
     *
     * @param string $token
     */
    public function sendPasswordResetNotification($token)
    {
        $resetUrl = URL::frontUrl('/reset-password', [
            'token' => $token,
            'email' => $this->email
        ]);

        $this->notify(new class($resetUrl) extends ResetPassword {
            protected $resetUrl;

            public function __construct($resetUrl)
            {
                $this->resetUrl = $resetUrl;
            }

            protected function resetUrl($notifiable)
            {
                return $this->resetUrl;
            }
        });
    }

    /**
     * Check if user is an admin
     *
     * @return bool
     */
    public function isAdmin()
    {
        return $this->user_type === 'admin';
    }

    /**
     * Check if user is a driver
     *
     * @return bool
     */
    public function isDriver()
    {
        return $this->user_type === 'motorista';
    }

    /**
     * Check if user is active
     *
     * @return bool
     */
    public function isActive()
    {
        return $this->is_active === true;
    }

    /**
     * Scope a query to only include active users
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopeActive($query)
    {
        return $query->where('is_active', true);
    }

    /**
     * Scope a query to only include admin users
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopeAdmins($query)
    {
        return $query->where('user_type', 'admin');
    }

    /**
     * Scope a query to only include drivers
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopeDrivers($query)
    {
        return $query->where('user_type', 'motorista');
    }

    /**
     * Get all fretes created by the user.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function fretes()
    {
        return $this->hasMany(Frete::class);
    }
}
